#!/bin/bash

# SPDX-FileCopyrightText: 2024 2024 Brandenburgische Technische Universität Cottbus-Senftenberg
#
# SPDX-License-Identifier: BSD-3-Clause

#######################################
# Extracts the height level for the variable from the given file 
# Globals:
#   None
# Arguments:
#   $1: File to retrieve height level from
# Outputs:
#   Outputs the found height level
# Returns:
#   None
#######################################
get_height () {
   feld=($(cdo zaxisdes $1))
#   search="units"
#   for i in "${!feld[@]}"; do
#      echo $i
#      echo ${feld[$i]}
#      [[ "${feld[$i]}" = "${search}" ]] && break
#   done
#   # TEST
#   if [ "${feld[$(($i + 2))]}" != "m" ]; then
#      echo "ERROR unit of height: ${feld[$(($i + 2))]}"
#      ###  exit script with error message exit /b
#   fi
   search="levels"
   for i in "${!feld[@]}"; do
      [[ "${feld[$i]}" = "${search}" ]] && break
   done
   ### to-do: abfrage nicht gefunden i== echo ${#feld[@]}
   echo ${feld[$(($i + 2))]}
}

#######################################
# Extracts the gridsize for the x axis of the given file
# Globals:
#   None
# Arguments:
#   $1: File to retrieve gridsize from
# Outputs:
#   Outputs the found x gridsize
# Returns:
#   None
#######################################
get_x_size () {
   feld=($(cdo griddes $1))
   search="xsize"
   for i in "${!feld[@]}"; do
      [[ "${feld[$i]}" = "${search}" ]] && break
   done
   echo ${feld[$(($i + 2))]}
   ### to-do: abfrage nicht gefunden i== echo ${#feld[@]}
}

#######################################
# Extracts the gridsize for the y axis of the given file
# Globals:
#   None
# Arguments:
#   $1: File to retrieve gridsize from
# Outputs:
#   Outputs the found y gridsize
# Returns:
#   None
#######################################
get_y_size () {
   feld=($(cdo griddes $1))
   search="ysize"
   for i in "${!feld[@]}"; do
      [[ "${feld[$i]}" = "${search}" ]] && break
   done
   echo ${feld[$(($i + 2))]}
   ### to-do: abfrage nicht gefunden i== echo ${#feld[@]}
}
### todo: unite 3 last functions

#######################################
# Converts unit of given dataset to scientific standard unit for given variable
# Globals:
#   None
# Arguments:
#   $1: File containing the data
#   $2: Variable name
# Outputs:
#   Converted data (values and unit) are saved to new file while echoing the done convertion to the user
# Returns:
#   None
#######################################
convert_unit() {
   UNIT=$( cdo partab $1 | grep "units" | grep -o -P '(?<=").*(?=")' )

   case ${UNIT} in
   "K" | "Kelvin" | "KELVIN")
      cdo -s -setmissval,nan -setattribute,$2@units="Celsius" -addc,-273.15 $1 $1'_new.nc'
      mv $1'_new.nc' $1
	   ;;
   "°C" | "Celsius" | "CELSIUS")
	   cdo -s -setmissval,nan $1 $1'_new.nc'
	   mv $1'_new.nc' $1
      ;;
   "km h-1")
      cdo -s -setmissval,nan -setattribute,$2@units="m s-1" -divc,3.6 $1 $1'_new.nc'
      mv $1'_new.nc' $1
      ;;
   "m s-1")
      cdo -s -setmissval,nan $1 $1'_new.nc'
      mv $1'_new.nc' $1
      ;;
   "Pa")
      cdo -s -setmissval,nan -setattribute,$2@units="kPa" -divc,1000 $1 $1'_new.nc'
      mv $1'_new.nc' $1
      ;;
   "kPa")
      cdo -s -setmissval,nan $1 $1'_new.nc'
      mv $1'_new.nc' $1
      ;;
   "W m-2")
      cdo -s -setmissval,nan -setattribute,$2@units="MJ m-2 d-1" -mulc,0.0864 $1 $1'_new.nc'    # 60*60*24 / 1000000
      mv $1'_new.nc' $1
      ;;
   "MJ m-2 d-1")
      cdo -s -setmissval,nan $1 $1'_new.nc'
      mv $1'_new.nc' $1
      ;;
   "%")
      cdo -s -setmissval,nan $1 $1'_new.nc'
      mv $1'_new.nc' $1
      ;;
   *)
      log_error "Error: The data doesnt match the given units (temperature: [°C]/[K], wind: [km h-1]/[m s-1], water vapour pressure: [Pa]/[kPa], radiation: [W m-2]/[MJ m-2 d-1])"
      exit 1 ### exit /b
   esac

   NEW_UNIT=$( cdo partab $1 | grep "units" | grep -o -P '(?<=").*(?=")' )
   log_debug "Converted unit... ${UNIT} -> ${NEW_UNIT}"
}
